function [cost , grad] = cnnCost(model, ffOutput, label, param)

numLayer = model.numLayer;
batchSize = size(label, 2);
weightDecay = param.weightDecay;
% assume the classification layer is softmax regression, compute the cost

cost = - mean( log(ffOutput{numLayer}.activation(label == 1) ) );
for l = 2 : numLayer
	if ~strcmp(model.layers{l}.type, 'subsampling')
		cost = cost + 0.5 * weightDecay * sum(model.layers{l}.weight(:).^ 2);
	end
end

% Initialize the error  adn grad for the whole network.
grad = cell(model.numLayer , 1);
error = cell(model.numLayer, 1);
for l = 2 : numLayer
	if strcmp( model.layers{l}.type, 'convolution')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.bias));
	elseif strcmp( model.layers{l}.type, 'subsampling')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.weight));
	elseif strcmp( model.layers{l}.type, 'classification')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
	elseif strcmp(model.layers{l}.type, 'fullconnected')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.bias));
	end
	error{l} = zeros(size(ffOutput{l}.activation)); % (rows, cols, batchSize, outputMaps) // ( hiddenUnits, batchSize)
end
% Initialize the output layer
error{numLayer} = - ( label - ffOutput{numLayer}.activation );

% Backpropagate the errors
for l = numLayer - 1 : -1 : 2
	if strcmp( model.layers{l+1}.type, 'classification' )
		if strcmp( model.layers{l}.type, 'fullconnected')
			error{l} = model.layers{l+1}.weight' * error{l+1};
		else
			thisLayerSize = model.layers{l}.layerSize;
			error{l} = permute( reshape( model.layers{l+1}.weight' * error{l+1}, [thisLayerSize batchSize]), [ 1 2 4 3]);
		end
		
	elseif strcmp( model.layers{l+1}.type, 'fullconnected')
		if strcmp( model.layers{l}.type , 'fullconnected')
			error{l} = model.layers{l+1}.weight' * error{l+1};
		else
			thisLayerSize = model.layers{l}.layerSize;
			error{l} = permute( reshape( model.layers{l+1}.weight' * error{l+1}, [thisLayerSize batchSize]), [ 1 2 4 3]);
		end
		
	elseif strcmp( model.layers{l+1}.type, 'convolution' )
		for o = 1 : model.layers{l+1}.layerSize(3)
			for i = 1 : model.layers{l}.layerSize(3)
				error{l}(:,:,:,i) = error{l}(:,:,:,i) + convn( error{l+1}(:,:,:,o), model.layers{l+1}.weight(:,:,i,o), 'full');
			end
		end
		
	elseif strcmp( model.layers{l+1}.type, 'subsampling' )
		scale = model.layers{l+1}.scale;
		for o = 1 : model.layers{l+1}.layerSize(3)
			for ex = 1 : batchSize
				error{l}(:,:,ex,o) = kron( error{l+1}(:,:,ex,o), ones(scale) ) .* model.layers{l+1}.weight(o) ./ scale .^ 2;
			end
		end	
	end
	% multiplying the derivatives of neuron's activation
	if strcmp( model.layers{l}.actFun , 'sigmoid')
		error{l} = error{l} .* ffOutput{l}.activation .* ( 1 - ffOutput{l}.activation );
	elseif strcmp( model.layers{l}.actFun, 'tanh')
		error{l} = error{l} .* ( 1 - ffOutput{l}.activation .^ 2 );
	else % identity function
		continue;
	end
end

% Compute the gradients for each layer
for l = 2 : numLayer
	if strcmp( model.layers{l}.type, 'convolution')
		grad{l}.bias = sum(reshape(error{l},[],size(error{l},4)),1)'./ batchSize; 
		for o = 1 : model.layers{l}.layerSize(3)
			for i = 1 : model.layers{l-1}.layerSize(3)
				thisError = flipdim(flipdim(flipdim(error{l}(:,:,:,o),1),2),3);
				grad{l}.weight(:,:,i,o) = convn(ffOutput{l-1}.activation(:,:,:,i), thisError, 'valid') ./ batchSize + weightDecay * model.layers{l}.weight(:,:,i,o);
			end
		end
	elseif strcmp( model.layers{l}.type, 'subsampling')
		grad{l}.bias = sum(reshape(error{l},[],size(error{l},4)),1)'./ batchSize; % mean is acting on examples
		grad{l}.weight = sum(reshape(ffOutput{l}.downSample .* error{l}, [], size(error{l},4)),1)' ./ batchSize;
	elseif strcmp( model.layers{l}.type, 'classification')
		if strcmp( model.layers{l-1}.type, 'fullconnected')
			grad{l}.weight = error{l} * ffOutput{l-1}.activation' ./ batchSize + weightDecay * model.layers{l}.weight;
		else
			preLayerActivation = reshape( permute(ffOutput{l-1}.activation, [1 2 4 3]), [], batchSize);
			grad{l}.weight = error{l} * preLayerActivation' ./ batchSize + weightDecay * model.layers{l}.weight; 
		end
	elseif strcmp( model.layers{l}.type, 'fullconnected')
		if strcmp(model.layers{l-1}.type, 'fullconnected')
			grad{l}.weight = error{l} * ffOutput{l-1}.activation' ./ batchSize + weightDecay * model.layers{l}.weight;
			grad{l}.bias = sum(error{l},2) ./ batchSize;
		else
			preLayerActivation = reshape( permute(ffOutput{l-1}.activation, [1 2 4 3]), [], batchSize);
			grad{l}.weight = error{l} * preLayerActivation' ./ batchSize + weightDecay * model.layers{l}.weight ;
			grad{l}.bias = sum(error{l},2) ./ batchSize;
		end
	end
end