function [output] = cnnFeedForward(model, data)

batchSize = size(data, 4);
output = cell(model.numLayer,1);
output{1}.activation = permute(data, [1 2 4 3]);
for l = 2 : model.numLayer
	output{l}.activation = permute( zeros([ model.layers{l}.layerSize batchSize ]), [1 2 4 3] );
end

for l = 2 : model.numLayer
	if strcmp(model.layers{l}.type , 'convolution')
		for o = 1 : model.layers{l}.layerSize(3)
			for i = 1 : model.layers{l-1}.layerSize(3)
				kernel = model.layers{l}.weight(:,:,i,o);
				kernel = rot90(squeeze(kernel),2);
				output{l}.activation(:,:,:,o) = output{l}.activation(:,:,:,o) + convn(squeeze( output{l-1}.activation(:,:,:,i) ), kernel, 'valid' );
			end
			output{l}.activation(:,:,:,o) = output{l}.activation(:,:,:,o) + model.layers{l}.bias(o);
		end
	elseif strcmp(model.layers{l}.type , 'subsampling')
		scale = model.layers{l}.scale;
		kernel = ones(scale) ./ scale .^ 2; % mean subsampling
		for o = 1 : model.layers{l}.layerSize(3)
			meanFilterMap = convn(squeeze(output{l-1}.activation(:,:,:,o)), kernel, 'valid');
			output{l}.downSample(:,:,:,o) = meanFilterMap(1:scale:model.layers{l-1}.layerSize(1)-scale+1 , 1:scale:model.layers{l-1}.layerSize(2)- scale+1,:);
			output{l}.activation(:,:,:,o) = output{l}.downSample(:,:,:,o) .* model.layers{l}.weight(o) + model.layers{l}.bias(o);
		end
	elseif strcmp(model.layers{l}.type, 'fullconnected')
		if strcmp ( model.layers{l-1}.type, 'fullconnected')
			output{l}.activation = bsxfun(@plus, model.layers{l}.weight * output{l-1}.activation, model.layers{l}.bias);
		else
			preLayerActivation = reshape( permute(output{l-1}.activation, [1 2 4 3]), [], batchSize); % (rows, cols, numMaps, batchSize)
			output{l}.activation = bsxfun(@plus, model.layers{l}.weight * preLayerActivation, model.layers{l}.bias);
		end
	elseif strcmp(model.layers{l}.type, 'classification')
		if strcmp (model.layers{l-1}.type, 'fullconnected')
			preLayerActivation = output{l-1}.activation;
		else
			preLayerActivation = reshape( permute(output{l-1}.activation, [1 2 4 3]), [], batchSize); % (rows, cols, numMaps, batchSize)
		end
		energy = exp( bsxfun (@minus, model.layers{l}.weight * preLayerActivation , max(model.layers{l}.weight * preLayerActivation, [], 1)));
		output{l}.activation = bsxfun(@rdivide, energy, sum(energy,1));
	end
	
	if strcmp(model.layers{l}.actFun, 'sigmoid')
		output{l}.activation = sigmoid(output{l}.activation);
	elseif strcmp(model.layers{l}.actFun, 'tanh')
		output{l}.activation = tanh(output{l}.activation);
	else
		continue;
	end
end

function Y = tanh(X)
    a = 1.7159;
    b = 2/3;
    Y = a * (exp(b*X) - exp(-b*X)) ./ (exp(b*X) + exp(-b*X));

function Y = sigmoid(X)
    Y = 1 ./ (1 + exp(-X));
