function [grad] = computeNumericalGradient(J, model)

numLayer = model.numLayer;

grad = cell(numLayer , 1);
for l = 2 : numLayer
	if strcmp( model.layers{l}.type, 'convolution')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.bias));
	elseif strcmp( model.layers{l}.type, 'subsampling')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.weight));
	elseif strcmp( model.layers{l}.type, 'classification')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
	elseif strcmp( model.layers{l}.type, 'fullconnected')
		grad{l}.weight = zeros(size(model.layers{l}.weight));
		grad{l}.bias = zeros(size(model.layers{l}.bias));
	end
end

epsilon = 1e-4;

for l = 2 : numLayer
	if strcmp( model.layers{l}.type, 'convolution')
		for i = 1 : numel(model.layers{l}.weight)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.weight(i) = modelLeft.layers{l}.weight(i) - epsilon;
			modelRight.layers{l}.weight(i) = modelRight.layers{l}.weight(i) + epsilon;
			grad{l}.weight(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		for i = 1 : numel(model.layers{l}.bias)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.bias(i) = modelLeft.layers{l}.bias(i) - epsilon;
			modelRight.layers{l}.bias(i) = modelRight.layers{l}.bias(i) + epsilon;
			grad{l}.bias(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		
	elseif strcmp( model.layers{l}.type, 'subsampling')
		for i = 1 : numel(model.layers{l}.weight)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.weight(i) = modelLeft.layers{l}.weight(i) - epsilon;
			modelRight.layers{l}.weight(i) = modelRight.layers{l}.weight(i) + epsilon;
			grad{l}.weight(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		for i = 1 : numel(model.layers{l}.bias)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.bias(i) = modelLeft.layers{l}.bias(i) - epsilon;
			modelRight.layers{l}.bias(i) = modelRight.layers{l}.bias(i) + epsilon;
			grad{l}.bias(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		
	elseif strcmp( model.layers{l}.type, 'classification')
		for i = 1 : numel(model.layers{l}.weight)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.weight(i) = modelLeft.layers{l}.weight(i) - epsilon;
			modelRight.layers{l}.weight(i) = modelRight.layers{l}.weight(i) + epsilon;
			grad{l}.weight(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		
	elseif strcmp( model.layers{l}.type, 'fullconnected')
		for i = 1 : numel(model.layers{l}.weight)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.weight(i) = modelLeft.layers{l}.weight(i) - epsilon;
			modelRight.layers{l}.weight(i) = modelRight.layers{l}.weight(i) + epsilon;
			grad{l}.weight(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
		for i = 1 : numel(model.layers{l}.bias)
			modelLeft = model;
			modelRight = model;
			modelLeft.layers{l}.bias(i) = modelLeft.layers{l}.bias(i) - epsilon;
			modelRight.layers{l}.bias(i) = modelRight.layers{l}.bias(i) + epsilon;
			grad{l}.bias(i) = (J(modelRight) - J(modelLeft)) ./ ( 2 * epsilon);
		end
	end
	
end


